@@ -78,15 +78,16 @@
 #include "vector_functions.h"
 #include "shared_header.h"
 #include "dice_logic.h"
+#include "safe_functions.h"
 #include "macro_logic.h"
 #include "rolls/sided_dice.h"
 #include "rolls/condition_checking.h"
+#include <errno.h>
 
 #define UNUSED(x) (void)(x)
 
 int yylex(void);
 int yyerror(const char* s);
-void print_err_if_present(int err_code);
 
 int yydebug=1;
 int verbose = 1;
@@ -94,10 +95,12 @@ int seeded = 0;
 int write_to_file = 0;
 char * output_file;
 
+extern unsigned int gnoll_errno;
+
 // Registers
 
 // TODO: It would be better to fit arbitrary length strings.
-unsigned int MAX_SYMBOL_TEXT_LENGTH = 100;
+unsigned int MAX_SYMBOL_TEXT_LENGTH = 256;
 unsigned int MAX_ITERATION = 20;
 
 int initialize(){
@@ -121,14 +124,14 @@ int sum(int * arr, int len){
 int roll_numeric_die(int small, int big){
     return random_fn(small, big);
 }
+
 int roll_symbolic_die(int length_of_symbolic_array){
     // Returns random index into symbolic array
     return roll_numeric_die(0, length_of_symbolic_array -1);
 }
 
 
-
-#line 132 "y.tab.c"
+#line 135 "y.tab.c"
 
 # ifndef YY_CAST
 #  ifdef __cplusplus
@@ -266,11 +269,11 @@ extern int yydebug;
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 union YYSTYPE
 {
-#line 89 "src/grammar/dice.yacc"
+#line 92 "src/grammar/dice.yacc"
 
     vec values;
 
-#line 274 "y.tab.c"
+#line 277 "y.tab.c"
 
 };
 typedef union YYSTYPE YYSTYPE;
@@ -649,13 +652,13 @@ static const yytype_int8 yytranslate[] =
   /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_int16 yyrline[] =
 {
-       0,    99,    99,   101,   104,   106,   111,   119,   164,   168,
-     193,   218,   243,   269,   318,   348,   372,   376,   386,   416,
-     450,   482,   499,   515,   531,   547,   566,   585,   599,   615,
-     630,   646,   652,   674,   698,   721,   744,   767,   791,   813,
-     836,   857,   882,   906,   932,   951,   973,   975,   979,  1000,
-    1048,  1062,  1080,  1105,  1108,  1108,  1108,  1108,  1108,  1108,
-    1111,  1118
+       0,   103,   103,   105,   108,   110,   115,   129,   182,   186,
+     212,   239,   269,   301,   363,   398,   427,   431,   441,   469,
+     506,   541,   561,   579,   590,   601,   614,   627,   636,   647,
+     657,   668,   674,   691,   709,   727,   744,   762,   780,   797,
+     815,   830,   849,   867,   887,   900,   916,   918,   922,   938,
+     980,   994,  1012,  1038,  1041,  1041,  1041,  1041,  1041,  1041,
+    1044,  1051
 };
 #endif
 
@@ -1515,74 +1518,88 @@ yyreduce:
   switch (yyn)
     {
   case 6:
-#line 111 "src/grammar/dice.yacc"
+#line 115 "src/grammar/dice.yacc"
                                                 {
-        // TODO: Is not recalculating if used twice.
+                
         vec key = (yyvsp[-2].values);
         vec value = (yyvsp[0].values);
-        register_macro(key.symbols[0], &value);
+
+        register_macro(&key, &value);
+        
+        if(gnoll_errno){
+            YYABORT;
+            yyclearin;
+        }
     }
-#line 1526 "y.tab.c"
+#line 1535 "y.tab.c"
     break;
 
   case 7:
-#line 119 "src/grammar/dice.yacc"
+#line 129 "src/grammar/dice.yacc"
                     {
 
-    vec vector;
-    vec new_vec;
-    vector = (yyvsp[0].values);
+    vec vector = (yyvsp[0].values);
+    vec new_vec = vector;       // Code Smell.
+                                // Target Vector should be empty
 
-    new_vec = vector;
     //  Step 1: Collapse pool to a single value if nessicary
     collapse_vector(&vector, &new_vec);
-
-    // Step 2: Output
-    FILE *fp;
-
-    if(write_to_file){
-        fp = fopen(output_file, "a+");
+    if(gnoll_errno){
+        YYABORT;
+        yyclearin;
     }
 
+    // Step 2: Output to file
+    FILE *fp = NULL;
+
+    if(write_to_file){
+        fp = safe_fopen(output_file, "a+");
+        if(gnoll_errno){
+            YYABORT;
+            yyclearin;
+        }
+    }
+
+    // TODO: To Function
     for(int i = 0; i!= new_vec.length;i++){
         if (new_vec.dtype == SYMBOLIC){
             // TODO: Strings >1 character
             if (verbose){
-                printf("%s;", new_vec.symbols[i]);
+                safe_printf("%s;", new_vec.symbols[i]);
             }
             if(write_to_file){
-                fprintf(fp, "%s;", new_vec.symbols[i]);
+                safe_fprintf(fp, "%s;", new_vec.symbols[i]);
             }
         }else{
             if(verbose){
-                printf("%d;", new_vec.content[i]);
+                safe_printf("%d;", new_vec.content[i]);
             }
             if(write_to_file){
-                fprintf(fp, "%d;", new_vec.content[i]);
+                safe_fprintf(fp, "%d;", new_vec.content[i]);
             }
         }
     }
     if(verbose){
-        printf("\n");
+       safe_printf("\n");
     }
 
     if(write_to_file){
-        fclose(fp);
+        safe_fclose(fp);
     }
 }
-#line 1574 "y.tab.c"
+#line 1591 "y.tab.c"
     break;
 
   case 8:
-#line 164 "src/grammar/dice.yacc"
+#line 182 "src/grammar/dice.yacc"
                       {
-        (yyval.values) =  (yyvsp[-1].values);
+        (yyval.values) = (yyvsp[-1].values);
     }
-#line 1582 "y.tab.c"
+#line 1599 "y.tab.c"
     break;
 
   case 9:
-#line 168 "src/grammar/dice.yacc"
+#line 186 "src/grammar/dice.yacc"
                   {
         // Collapse both sides and subtract
         vec vector1;
@@ -1591,7 +1608,8 @@ yyreduce:
         vector2 = (yyvsp[0].values);
 
         if (vector1.dtype == SYMBOLIC || vector2.dtype == SYMBOLIC){
-            printf("Division unsupported for symbolic dice.");
+            safe_printf("Multiplication not implemented for symbolic dice.\n");
+            gnoll_errno = NOT_IMPLEMENTED;
             YYABORT;
             yyclearin;
         }else{
@@ -1599,7 +1617,7 @@ yyreduce:
             int v2 = collapse(vector2.content, vector2.length);
 
             vec new_vec;
-            new_vec.content = calloc(sizeof(int), 1);
+            new_vec.content = safe_calloc(sizeof(int), 1);
             new_vec.length = 1;
             new_vec.content[0] = v1 * v2;
             new_vec.dtype = vector1.dtype;
@@ -1607,11 +1625,11 @@ yyreduce:
             (yyval.values) = new_vec;
         }
     }
-#line 1611 "y.tab.c"
+#line 1629 "y.tab.c"
     break;
 
   case 10:
-#line 193 "src/grammar/dice.yacc"
+#line 212 "src/grammar/dice.yacc"
                              {
         // Collapse both sides and subtract
         vec vector1;
@@ -1620,15 +1638,17 @@ yyreduce:
         vector2 = (yyvsp[0].values);
 
         if (vector1.dtype == SYMBOLIC || vector2.dtype == SYMBOLIC){
-            printf("Division unsupported for symbolic dice.");
+            safe_printf("Division unsupported for symbolic dice.\n");
+            gnoll_errno = UNDEFINED_BEHAVIOUR;
             YYABORT;
             yyclearin;
+
         }else{
             int v1 = collapse(vector1.content, vector1.length);
             int v2 = collapse(vector2.content, vector2.length);
 
             vec new_vec;
-            new_vec.content = calloc(sizeof(int), 1);
+            new_vec.content = safe_calloc(sizeof(int), 1);
             new_vec.length = 1;
             new_vec.content[0] = (v1+(v2-1))/ v2;
             new_vec.dtype = vector1.dtype;
@@ -1636,11 +1656,11 @@ yyreduce:
             (yyval.values) = new_vec;
         }
     }
-#line 1640 "y.tab.c"
+#line 1660 "y.tab.c"
     break;
 
   case 11:
-#line 218 "src/grammar/dice.yacc"
+#line 239 "src/grammar/dice.yacc"
                                {
         // Collapse both sides and subtract
         vec vector1;
@@ -1649,7 +1669,8 @@ yyreduce:
         vector2 = (yyvsp[0].values);
 
         if (vector1.dtype == SYMBOLIC || vector2.dtype == SYMBOLIC){
-            printf("Modulo unsupported for symbolic dice.");
+            safe_printf("Division unsupported for symbolic dice.\n");
+            gnoll_errno = UNDEFINED_BEHAVIOUR;
             YYABORT;
             yyclearin;
         }else{
@@ -1657,7 +1678,11 @@ yyreduce:
             int v2 = collapse(vector2.content, vector2.length);
 
             vec new_vec;
-            new_vec.content = calloc(sizeof(int), 1);
+            new_vec.content = safe_calloc(sizeof(int), 1);
+            if(gnoll_errno){
+               YYABORT;
+               yyclearin;
+            }
             new_vec.length = 1;
             new_vec.content[0] = v1 / v2;
             new_vec.dtype = vector1.dtype;
@@ -1665,11 +1690,11 @@ yyreduce:
             (yyval.values) = new_vec;
         }
     }
-#line 1669 "y.tab.c"
+#line 1694 "y.tab.c"
     break;
 
   case 12:
-#line 243 "src/grammar/dice.yacc"
+#line 269 "src/grammar/dice.yacc"
                     {
         // Collapse both sides and subtract
         vec vector1;
@@ -1679,15 +1704,21 @@ yyreduce:
         vector2 = (yyvsp[0].values);
 
         if (vector1.dtype == SYMBOLIC || vector2.dtype == SYMBOLIC){
-            printf("Modulo unsupported for symbolic dice.");
+            safe_printf("Modulo unsupported for symbolic dice.\n");
+            gnoll_errno = UNDEFINED_BEHAVIOUR;
             YYABORT;
             yyclearin;
+
         }else{
             int v1 = collapse(vector1.content, vector1.length);
             int v2 = collapse(vector2.content, vector2.length);
 
             vec new_vec;
-            new_vec.content = calloc(sizeof(int), 1);
+            new_vec.content = safe_calloc(sizeof(int), 1);
+            if(gnoll_errno){
+                YYABORT;
+                yyclearin;
+            }
             new_vec.length = 1;
             new_vec.content[0] = v1 % v2;
             new_vec.dtype = vector1.dtype;
@@ -1695,11 +1726,11 @@ yyreduce:
             (yyval.values) = new_vec;
         }
     }
-#line 1699 "y.tab.c"
+#line 1730 "y.tab.c"
     break;
 
   case 13:
-#line 269 "src/grammar/dice.yacc"
+#line 301 "src/grammar/dice.yacc"
                   {
         // Collapse both sides and subtract
         vec vector1;
@@ -1711,15 +1742,24 @@ yyreduce:
             (vector1.dtype == SYMBOLIC && vector2.dtype == NUMERIC) ||
             (vector2.dtype == SYMBOLIC && vector1.dtype == NUMERIC)
         ){
-            printf("Addition not supported with mixed dice types.");
+            safe_printf("Addition not supported with mixed dice types.\n");
+            gnoll_errno = UNDEFINED_BEHAVIOUR;
             YYABORT;
             yyclearin;
         } else if (vector1.dtype == SYMBOLIC){
             vec new_vec;
             unsigned int concat_length = vector1.length + vector2.length;
-            new_vec.symbols = calloc(sizeof(char *), concat_length);
+            new_vec.symbols = safe_calloc(sizeof(char *), concat_length);
+            if(gnoll_errno){
+                YYABORT;
+                yyclearin;
+            }
             for (int i = 0; i != concat_length; i++){
-                new_vec.symbols[i] = calloc(sizeof(char), MAX_SYMBOL_TEXT_LENGTH);
+                new_vec.symbols[i] = safe_calloc(sizeof(char), MAX_SYMBOL_TEXT_LENGTH);
+                if(gnoll_errno){
+                    YYABORT;
+                    yyclearin;
+                }
             }
             new_vec.length = concat_length;
             new_vec.dtype = vector1.dtype;
@@ -1739,7 +1779,11 @@ yyreduce:
             int v2 = collapse(vector2.content, vector2.length);
 
             vec new_vec;
-            new_vec.content = calloc(sizeof(int), 1);
+            new_vec.content = safe_calloc(sizeof(int), 1);
+            if(gnoll_errno){
+                YYABORT;
+                yyclearin;
+            }
             new_vec.length = 1;
             new_vec.dtype = vector1.dtype;
             new_vec.content[0] = v1 + v2;
@@ -1748,11 +1792,11 @@ yyreduce:
         }
 
     }
-#line 1752 "y.tab.c"
+#line 1796 "y.tab.c"
     break;
 
   case 14:
-#line 318 "src/grammar/dice.yacc"
+#line 363 "src/grammar/dice.yacc"
                    {
         vec vector1;
         vec vector2;
@@ -1763,9 +1807,10 @@ yyreduce:
         ){
             // It's not clear whether {+,-} - {-, 0} should be {+} or {+, 0}!
             // Therfore, we'll exclude it.
-            printf("Subtract not supported with symbolic dice.");
+            safe_printf("Subtract not supported with symbolic dice.\n");
+            gnoll_errno = UNDEFINED_BEHAVIOUR;
             YYABORT;
-            yyclearin;
+            yyclearin;;
         }else{
             // Collapse both sides and subtract
 
@@ -1773,7 +1818,11 @@ yyreduce:
             int v2 = collapse(vector2.content, vector2.length);
 
             vec new_vec;
-            new_vec.content = calloc(sizeof(int), 1);
+            new_vec.content = safe_calloc(sizeof(int), 1);
+            if(gnoll_errno){
+                YYABORT;
+                yyclearin;
+            }
             new_vec.length = 1;
             new_vec.content[0] = v1 - v2;
             new_vec.dtype = vector1.dtype;
@@ -1782,24 +1831,29 @@ yyreduce:
         }
 
     }
-#line 1786 "y.tab.c"
+#line 1835 "y.tab.c"
     break;
 
   case 15:
-#line 348 "src/grammar/dice.yacc"
+#line 398 "src/grammar/dice.yacc"
                            {
         // Eltwise Negation
         vec vector;
         vector = (yyvsp[0].values);
 
         if (vector.dtype == SYMBOLIC){
-            printf("Symbolic Dice, Cannot negate. Consider using Numeric dice or post-processing.");
+            safe_printf("Symbolic Dice, Cannot negate. Consider using Numeric dice or post-processing.\n");
+            gnoll_errno = UNDEFINED_BEHAVIOUR;
             YYABORT;
-            yyclearin;
+            yyclearin;;
         } else {
             vec new_vec;
 
-            new_vec.content = calloc(sizeof(int), vector.length);
+            new_vec.content = safe_calloc(sizeof(int), vector.length);
+            if(gnoll_errno){
+                YYABORT;
+                yyclearin;
+            }
             new_vec.length = vector.length;
             new_vec.dtype = vector.dtype;
 
@@ -1810,11 +1864,11 @@ yyreduce:
 
         }
     }
-#line 1814 "y.tab.c"
+#line 1868 "y.tab.c"
     break;
 
   case 17:
-#line 376 "src/grammar/dice.yacc"
+#line 431 "src/grammar/dice.yacc"
                             {
 
         vec new_vec;
@@ -1824,11 +1878,11 @@ yyreduce:
         new_vec.content[0] = dice.length;
         (yyval.values) = new_vec;
     }
-#line 1828 "y.tab.c"
+#line 1882 "y.tab.c"
     break;
 
   case 18:
-#line 386 "src/grammar/dice.yacc"
+#line 441 "src/grammar/dice.yacc"
                    {
 
         vec vector;
@@ -1839,10 +1893,8 @@ yyreduce:
             (yyval.values) = vector;
         }
         else{
-            // Collapse if Nessicary
+            // Collapse if Necessary
             if(vector.length > 1){
-                int result = sum(vector.content, vector.length);
-
                 vec new_vector;
                 initialize_vector(&new_vector, NUMERIC, 1);
                 new_vector.content[0] = sum(vector.content, vector.length);
@@ -1854,11 +1906,11 @@ yyreduce:
 
         }
     }
-#line 1858 "y.tab.c"
+#line 1910 "y.tab.c"
     break;
 
   case 19:
-#line 416 "src/grammar/dice.yacc"
+#line 469 "src/grammar/dice.yacc"
                                            {
 
         vec dice = (yyvsp[-4].values);
@@ -1868,7 +1920,7 @@ yyreduce:
             int count = 0;
             while (! check_condition(&dice, &(yyvsp[0].values), check)){
                 if (count > MAX_ITERATION){
-                    printf("MAX ITERATION LIMIT EXCEEDED: REROLL");
+                    safe_printf("MAX ITERATION LIMIT EXCEEDED: REROLL\n");
                     break;
                 }
                 vec number_of_dice;
@@ -1890,14 +1942,17 @@ yyreduce:
             }
             (yyval.values) = dice;
         }else{
-            printf("No support for Symbolic die rerolling yet!");
+            safe_printf("No support for Symbolic die rerolling yet!\n");
+            gnoll_errno = NOT_IMPLEMENTED;
+            YYABORT;
+            yyclearin;;
         }
     }
-#line 1897 "y.tab.c"
+#line 1952 "y.tab.c"
     break;
 
   case 20:
-#line 450 "src/grammar/dice.yacc"
+#line 506 "src/grammar/dice.yacc"
                                      {
 
         vec dice = (yyvsp[-3].values);
@@ -1926,14 +1981,17 @@ yyreduce:
                 (yyval.values) = (yyvsp[-3].values);
             }
         }else{
-            printf("No support for Symbolic die rerolling yet!");
+            safe_printf("No support for Symbolic die rerolling yet!");
+            gnoll_errno = NOT_IMPLEMENTED;
+            YYABORT;
+            yyclearin;;
         }
     }
-#line 1933 "y.tab.c"
+#line 1991 "y.tab.c"
     break;
 
   case 21:
-#line 482 "src/grammar/dice.yacc"
+#line 541 "src/grammar/dice.yacc"
                                            {
         vec new_vec;
         vec dice = (yyvsp[-3].values);
@@ -1946,19 +2004,21 @@ yyreduce:
 
             (yyval.values) = new_vec;
         }else{
-            printf("No support for Symbolic die rerolling yet!");
+            safe_printf("No support for Symbolic die rerolling yet!\n");
+            gnoll_errno = NOT_IMPLEMENTED;
+            YYABORT;
+            yyclearin;;
         }
 
     }
-#line 1954 "y.tab.c"
+#line 2015 "y.tab.c"
     break;
 
   case 22:
-#line 499 "src/grammar/dice.yacc"
+#line 561 "src/grammar/dice.yacc"
                                {
         // TODO
         vec new_vec;
-        vec cond_vec;
         vec dice = (yyvsp[-1].values);
 
         if(dice.dtype == NUMERIC){
@@ -1967,176 +2027,137 @@ yyreduce:
 
             (yyval.values) = new_vec;
         }else{
-            printf("No support for Symbolic die rerolling yet!");
+            safe_printf("No support for Symbolic die rerolling yet!\n");
+            gnoll_errno = NOT_IMPLEMENTED;
+            YYABORT;
+            yyclearin;;
         }
     }
-#line 1974 "y.tab.c"
+#line 2037 "y.tab.c"
     break;
 
   case 23:
-#line 516 "src/grammar/dice.yacc"
+#line 580 "src/grammar/dice.yacc"
     {
         vec keep_vector = (yyvsp[0].values);
         vec new_vec;
         unsigned int num_to_hold = keep_vector.content[0];
 
-        unsigned int err = keep_highest_values(&(yyvsp[-2].values), &new_vec, num_to_hold);
+        keep_highest_values(&(yyvsp[-2].values), &new_vec, num_to_hold);
 
-        if(err){
-            printf("Error in: KeepHighestN.");
-            YYABORT;
-            yyclearin;
-        }
         (yyval.values) = new_vec;
     }
-#line 1993 "y.tab.c"
+#line 2051 "y.tab.c"
     break;
 
   case 24:
-#line 532 "src/grammar/dice.yacc"
+#line 591 "src/grammar/dice.yacc"
     {
         vec keep_vector = (yyvsp[0].values);
         vec new_vec;
         unsigned int num_to_hold = keep_vector.content[0];
 
-        unsigned int err = drop_highest_values(&(yyvsp[-2].values), &new_vec, num_to_hold);
+        drop_highest_values(&(yyvsp[-2].values), &new_vec, num_to_hold);
 
-        if(err){
-            printf("Error in: KeepHighestN.");
-            YYABORT;
-            yyclearin;
-        }
         (yyval.values) = new_vec;
     }
-#line 2012 "y.tab.c"
+#line 2065 "y.tab.c"
     break;
 
   case 25:
-#line 548 "src/grammar/dice.yacc"
+#line 602 "src/grammar/dice.yacc"
     {
-        vec roll_vector;
         vec keep_vector;
         unsigned int num_to_hold;
         keep_vector = (yyvsp[0].values);
         num_to_hold = keep_vector.content[0];
 
         vec new_vec;
-        unsigned int err = keep_lowest_values(&(yyvsp[-2].values), &new_vec, num_to_hold);
+        keep_lowest_values(&(yyvsp[-2].values), &new_vec, num_to_hold);
 
-        if(err){
-            printf("Error in: KeepLowestN.");
-            YYABORT;
-            yyclearin;
-        }
         (yyval.values) = new_vec;
     }
-#line 2034 "y.tab.c"
+#line 2081 "y.tab.c"
     break;
 
   case 26:
-#line 567 "src/grammar/dice.yacc"
+#line 615 "src/grammar/dice.yacc"
     {
-        vec roll_vector;
         vec keep_vector;
         unsigned int num_to_hold;
         keep_vector = (yyvsp[0].values);
         num_to_hold = keep_vector.content[0];
 
         vec new_vec;
-        unsigned int err = drop_lowest_values(&(yyvsp[-2].values), &new_vec, num_to_hold);
+        drop_lowest_values(&(yyvsp[-2].values), &new_vec, num_to_hold);
 
-        if(err){
-            printf("Error in: KeepLowestN.");
-            YYABORT;
-            yyclearin;
-        }
         (yyval.values) = new_vec;
     }
-#line 2056 "y.tab.c"
+#line 2097 "y.tab.c"
     break;
 
   case 27:
-#line 586 "src/grammar/dice.yacc"
+#line 628 "src/grammar/dice.yacc"
     {
         unsigned int num_to_hold = 1;
         vec new_vec;
-        unsigned int err = keep_highest_values(&(yyvsp[-1].values), &new_vec, num_to_hold);
+        keep_highest_values(&(yyvsp[-1].values), &new_vec, num_to_hold);
 
-        if(err){
-            printf("Error in: KeepHighest1.");
-            YYABORT;
-            yyclearin;
-        }
         (yyval.values) = new_vec;
     }
-#line 2073 "y.tab.c"
+#line 2109 "y.tab.c"
     break;
 
   case 28:
-#line 600 "src/grammar/dice.yacc"
+#line 637 "src/grammar/dice.yacc"
     {
         vec roll_vec = (yyvsp[-1].values);
         unsigned int num_to_hold = 1;
 
         vec new_vec;
-        unsigned int err = drop_highest_values(&roll_vec, &new_vec, num_to_hold);
+        drop_highest_values(&roll_vec, &new_vec, num_to_hold);
 
-        if(err){
-            printf("Error in: KeepHighest1.");
-            YYABORT;
-            yyclearin;
-        }
         (yyval.values) = new_vec;
     }
-#line 2092 "y.tab.c"
+#line 2123 "y.tab.c"
     break;
 
   case 29:
-#line 616 "src/grammar/dice.yacc"
+#line 648 "src/grammar/dice.yacc"
     {
         unsigned int num_to_hold = 1;
 
         vec new_vec;
-        unsigned int err = keep_lowest_values(&(yyvsp[-1].values), &new_vec, num_to_hold);
+        keep_lowest_values(&(yyvsp[-1].values), &new_vec, num_to_hold);
 
-        if(err){
-            printf("Error in: KeepHighest1.");
-            YYABORT;
-            yyclearin;
-        }
         (yyval.values) = new_vec;
     }
-#line 2110 "y.tab.c"
-    break;
-
-  case 30:
-#line 631 "src/grammar/dice.yacc"
-    {
-        vec roll_vec = (yyvsp[-1].values);
-        unsigned int num_to_hold = 1;
-
-        vec new_vec;
-        unsigned int err = drop_lowest_values(&roll_vec, &new_vec, num_to_hold);
-
-        if(err){
-            printf("Error in: KeepHighest1.");
-            YYABORT;
-            yyclearin;
-        }
-        (yyval.values) = new_vec;
-    }
-#line 2129 "y.tab.c"
-    break;
-
-  case 31:
-#line 647 "src/grammar/dice.yacc"
-    {
-    }
 #line 2136 "y.tab.c"
     break;
 
+  case 30:
+#line 658 "src/grammar/dice.yacc"
+    {
+        vec roll_vec = (yyvsp[-1].values);
+        unsigned int num_to_hold = 1;
+
+        vec new_vec;
+        drop_lowest_values(&roll_vec, &new_vec, num_to_hold);
+
+        (yyval.values) = new_vec;
+    }
+#line 2150 "y.tab.c"
+    break;
+
+  case 31:
+#line 669 "src/grammar/dice.yacc"
+    {
+    }
+#line 2157 "y.tab.c"
+    break;
+
   case 32:
-#line 653 "src/grammar/dice.yacc"
+#line 675 "src/grammar/dice.yacc"
     {
         int start_from = (yyvsp[-3].values).content[0];
 
@@ -2144,24 +2165,19 @@ yyreduce:
         initialize_vector(&number_of_dice, NUMERIC, 1);
         number_of_dice.content[0] = 1;
 
-        int err = roll_plain_sided_dice(
+        roll_plain_sided_dice(
             &(yyvsp[-4].values),
             &(yyvsp[-2].values),
             &(yyval.values),
             ONLY_ONCE_EXPLOSION,
             start_from
         );
-        print_err_if_present(err);
-        if(err){
-            YYABORT;
-            yyclearin;
-        }
     }
-#line 2161 "y.tab.c"
+#line 2177 "y.tab.c"
     break;
 
   case 33:
-#line 675 "src/grammar/dice.yacc"
+#line 692 "src/grammar/dice.yacc"
     {
 
         int start_from = (yyvsp[-3].values).content[0];
@@ -2170,25 +2186,19 @@ yyreduce:
         initialize_vector(&number_of_dice, NUMERIC, 1);
         number_of_dice.content[0] = 1;
 
-        int err = roll_plain_sided_dice(
+        roll_plain_sided_dice(
             &number_of_dice,
             &(yyvsp[-2].values),
             &(yyval.values),
             ONLY_ONCE_EXPLOSION,
             start_from
         );
-        print_err_if_present(err);
-        if(err){
-            YYABORT;
-            yyclearin;
-        }
-
     }
-#line 2188 "y.tab.c"
+#line 2198 "y.tab.c"
     break;
 
   case 34:
-#line 699 "src/grammar/dice.yacc"
+#line 710 "src/grammar/dice.yacc"
     {
 
         int start_from = (yyvsp[-3].values).content[0];
@@ -2197,24 +2207,19 @@ yyreduce:
         initialize_vector(&number_of_dice, NUMERIC, 1);
         number_of_dice.content[0] = 1;
 
-        int err = roll_plain_sided_dice(
+        roll_plain_sided_dice(
             &(yyvsp[-4].values),
             &(yyvsp[-2].values),
             &(yyval.values),
             PENETRATING_EXPLOSION,
             start_from
         );
-        print_err_if_present(err);
-        if(err){
-            YYABORT;
-            yyclearin;
-        }
     }
-#line 2214 "y.tab.c"
+#line 2219 "y.tab.c"
     break;
 
   case 35:
-#line 722 "src/grammar/dice.yacc"
+#line 728 "src/grammar/dice.yacc"
     {
         int start_from = (yyvsp[-3].values).content[0];
 
@@ -2222,21 +2227,15 @@ yyreduce:
         initialize_vector(&number_of_dice, NUMERIC, 1);
         number_of_dice.content[0] = 1;
 
-        int err = roll_plain_sided_dice(
+        roll_plain_sided_dice(
             &number_of_dice,
             &(yyvsp[-2].values),
             &(yyval.values),
             PENETRATING_EXPLOSION,
             start_from
         );
-        print_err_if_present(err);
-        if(err){
-            YYABORT;
-            yyclearin;
-        }
-
     }
-#line 2240 "y.tab.c"
+#line 2239 "y.tab.c"
     break;
 
   case 36:
@@ -2249,24 +2248,19 @@ yyreduce:
         initialize_vector(&number_of_dice, NUMERIC, 1);
         number_of_dice.content[0] = 1;
 
-        int err = roll_plain_sided_dice(
+        roll_plain_sided_dice(
             &(yyvsp[-3].values),
             &(yyvsp[-1].values),
             &(yyval.values),
             PENETRATING_EXPLOSION,
             start_from
         );
-        print_err_if_present(err);
-        if(err){
-            YYABORT;
-            yyclearin;
-        }
     }
-#line 2266 "y.tab.c"
+#line 2260 "y.tab.c"
     break;
 
   case 37:
-#line 768 "src/grammar/dice.yacc"
+#line 763 "src/grammar/dice.yacc"
     {
 
         int start_from = (yyvsp[-2].values).content[0];
@@ -2275,25 +2269,19 @@ yyreduce:
         initialize_vector(&number_of_dice, NUMERIC, 1);
         number_of_dice.content[0] = 1;
 
-        int err = roll_plain_sided_dice(
+        roll_plain_sided_dice(
             &number_of_dice,
             &(yyvsp[-1].values),
             &(yyval.values),
             STANDARD_EXPLOSION,
             start_from
         );
-        print_err_if_present(err);
-        if(err){
-            YYABORT;
-            yyclearin;
-        }
-
     }
-#line 2293 "y.tab.c"
+#line 2281 "y.tab.c"
     break;
 
   case 38:
-#line 792 "src/grammar/dice.yacc"
+#line 781 "src/grammar/dice.yacc"
     {
         int start_from = (yyvsp[-1].values).content[0];
 
@@ -2301,24 +2289,19 @@ yyreduce:
         initialize_vector(&number_of_dice, NUMERIC, 1);
         number_of_dice.content[0] = 1;
 
-        int err = roll_plain_sided_dice(
+        roll_plain_sided_dice(
             &(yyvsp[-2].values),
             &(yyvsp[0].values),
             &(yyval.values),
             NO_EXPLOSION,
             start_from
         );
-        print_err_if_present(err);
-        if(err){
-            YYABORT;
-            yyclearin;
-        }
     }
-#line 2318 "y.tab.c"
+#line 2301 "y.tab.c"
     break;
 
   case 39:
-#line 814 "src/grammar/dice.yacc"
+#line 798 "src/grammar/dice.yacc"
     {
 
         int start_from = (yyvsp[-1].values).content[0];
@@ -2327,48 +2310,37 @@ yyreduce:
         initialize_vector(&number_of_dice, NUMERIC, 1);
         number_of_dice.content[0] = 1;
 
-        int err = roll_plain_sided_dice(
+        roll_plain_sided_dice(
             &number_of_dice,
             &(yyvsp[0].values),
             &(yyval.values),
             NO_EXPLOSION,
             start_from
         );
-        print_err_if_present(err);
-        if(err){
-            YYABORT;
-            yyclearin;
-        }
     }
-#line 2344 "y.tab.c"
+#line 2322 "y.tab.c"
     break;
 
   case 40:
-#line 837 "src/grammar/dice.yacc"
+#line 816 "src/grammar/dice.yacc"
     {
         vec dice_sides;
         initialize_vector(&dice_sides, NUMERIC, 1);
         dice_sides.content[0] = 100;
 
-        int err = roll_plain_sided_dice(
+        roll_plain_sided_dice(
             &(yyvsp[-2].values),
             &dice_sides,
             &(yyval.values),
             NO_EXPLOSION,
             1
         );
-        print_err_if_present(err);
-        if(err){
-            YYABORT;
-            yyclearin;
-        }
-
     }
-#line 2368 "y.tab.c"
+#line 2340 "y.tab.c"
     break;
 
   case 41:
-#line 858 "src/grammar/dice.yacc"
+#line 831 "src/grammar/dice.yacc"
     {
 
         vec num_dice;
@@ -2378,25 +2350,19 @@ yyreduce:
         initialize_vector(&dice_sides, NUMERIC, 1);
         dice_sides.content[0] = 100;
 
-        int err = roll_plain_sided_dice(
+        roll_plain_sided_dice(
             &num_dice,
             &dice_sides,
             &(yyval.values),
             NO_EXPLOSION,
             1
         );
-        print_err_if_present(err);
-        if(err){
-            YYABORT;
-            yyclearin;
-        }
-
     }
-#line 2396 "y.tab.c"
+#line 2362 "y.tab.c"
     break;
 
   case 42:
-#line 883 "src/grammar/dice.yacc"
+#line 850 "src/grammar/dice.yacc"
     {
 
         int start_from = (yyvsp[-1].values).content[0];
@@ -2405,25 +2371,19 @@ yyreduce:
         initialize_vector(&dice_sides, NUMERIC, 1);
         dice_sides.content[0] = 2;
 
-        int err = roll_plain_sided_dice(
+        roll_plain_sided_dice(
             &(yyvsp[-2].values),
             &dice_sides,
             &(yyval.values),
             NO_EXPLOSION,
             start_from
         );
-        print_err_if_present(err);
-        if(err){
-            YYABORT;
-            yyclearin;
-        }
-
     }
-#line 2423 "y.tab.c"
+#line 2383 "y.tab.c"
     break;
 
   case 43:
-#line 907 "src/grammar/dice.yacc"
+#line 868 "src/grammar/dice.yacc"
     {
         int start_from = (yyvsp[-1].values).content[0];
 
@@ -2434,47 +2394,35 @@ yyreduce:
         initialize_vector(&dice_sides, NUMERIC, 1);
         dice_sides.content[0] = 2;
 
-        int err = roll_plain_sided_dice(
+        roll_plain_sided_dice(
             &num_dice,
             &dice_sides,
             &(yyval.values),
             NO_EXPLOSION,
             start_from
         );
-        print_err_if_present(err);
-        if(err){
-            YYABORT;
-            yyclearin;
-        }
-
     }
-#line 2452 "y.tab.c"
+#line 2406 "y.tab.c"
     break;
 
   case 44:
-#line 933 "src/grammar/dice.yacc"
+#line 888 "src/grammar/dice.yacc"
     {
         vec result_vec;
         initialize_vector(&result_vec, SYMBOLIC, (yyvsp[-1].values).content[0]);
 
-        int err = roll_symbolic_dice(
+        roll_symbolic_dice(
             &(yyvsp[-1].values),
             &(yyvsp[0].values),
             &result_vec
         );
         (yyval.values) = result_vec;
-        print_err_if_present(err);
-        if(err){
-            YYABORT;
-            yyclearin;
-        }
-
     }
-#line 2474 "y.tab.c"
+#line 2422 "y.tab.c"
     break;
 
   case 45:
-#line 952 "src/grammar/dice.yacc"
+#line 901 "src/grammar/dice.yacc"
     {
         vec result_vec;
         vec number_of_dice;
@@ -2482,24 +2430,18 @@ yyreduce:
         initialize_vector(&number_of_dice, NUMERIC, 1);
         number_of_dice.content[0] = 1;
 
-        int err = roll_symbolic_dice(
+        roll_symbolic_dice(
             &number_of_dice,
             &(yyvsp[0].values),
             &result_vec
         );
         (yyval.values) = result_vec;
-
-        print_err_if_present(err);
-        if(err){
-            YYABORT;
-            yyclearin;
-        }
     }
-#line 2499 "y.tab.c"
+#line 2441 "y.tab.c"
     break;
 
   case 48:
-#line 980 "src/grammar/dice.yacc"
+#line 923 "src/grammar/dice.yacc"
     {
 
         // TODO: Multiple ranges
@@ -2507,23 +2449,18 @@ yyreduce:
         vec result_vec;
         initialize_vector(&result_vec, SYMBOLIC, (yyvsp[-4].values).content[0]);
 
-        int err = roll_symbolic_dice(
+        roll_symbolic_dice(
             &(yyvsp[-4].values),
             &(yyvsp[-1].values),
             &result_vec
         );
         (yyval.values) = result_vec;
-        print_err_if_present(err);
-        if(err){
-            YYABORT;
-            yyclearin;
-        }
     }
-#line 2523 "y.tab.c"
+#line 2460 "y.tab.c"
     break;
 
   case 49:
-#line 1001 "src/grammar/dice.yacc"
+#line 939 "src/grammar/dice.yacc"
     {
         vec csd = (yyvsp[-1].values);
         vec result_vec;
@@ -2531,8 +2468,7 @@ yyreduce:
         initialize_vector(&number_of_dice, NUMERIC, 1);
         number_of_dice.content[0] = 1;
 
-        int err = 0;
-
+        
         if (csd.dtype == NUMERIC){
             vec dice_sides;
             vec num_dice;
@@ -2546,7 +2482,7 @@ yyreduce:
             dice_sides.content[0] = end_value - start_value + 1;
 
             // Range
-            err = roll_plain_sided_dice(
+            roll_plain_sided_dice(
                 &num_dice,
                 &dice_sides,
                 &result_vec,
@@ -2557,24 +2493,19 @@ yyreduce:
         }else{
             initialize_vector(&result_vec, SYMBOLIC, 1);
             // Custom Symbol
-            err = roll_symbolic_dice(
+            roll_symbolic_dice(
                 &number_of_dice,
                 &(yyvsp[-1].values),
                 &result_vec
             );
         }
         (yyval.values) = result_vec;
-        print_err_if_present(err);
-        if(err){
-            YYABORT;
-            yyclearin;
-        }
     }
-#line 2574 "y.tab.c"
+#line 2505 "y.tab.c"
     break;
 
   case 50:
-#line 1048 "src/grammar/dice.yacc"
+#line 980 "src/grammar/dice.yacc"
                                  {
         vec vector;
         vector = (yyvsp[0].values);
@@ -2587,11 +2518,11 @@ yyreduce:
 
         (yyval.values) = new_vector;
     }
-#line 2591 "y.tab.c"
+#line 2522 "y.tab.c"
     break;
 
   case 51:
-#line 1062 "src/grammar/dice.yacc"
+#line 994 "src/grammar/dice.yacc"
                                        {
         vec l;
         vec r;
@@ -2609,11 +2540,11 @@ yyreduce:
         (yyval.values) = new_vector;
 
     }
-#line 2613 "y.tab.c"
+#line 2544 "y.tab.c"
     break;
 
   case 52:
-#line 1080 "src/grammar/dice.yacc"
+#line 1012 "src/grammar/dice.yacc"
                        {
         vec start = (yyvsp[-2].values);
         vec end = (yyvsp[0].values);
@@ -2624,7 +2555,8 @@ yyreduce:
 
         if (s > e){
             printf("Range: %i -> %i\n", s, e);
-            printf("Reversed Ranged not supported yet.");
+            printf("Reversed Ranged not supported yet.\n");
+            gnoll_errno = NOT_IMPLEMENTED;
             YYABORT;
             yyclearin;
         }
@@ -2638,33 +2570,33 @@ yyreduce:
         }
         (yyval.values) = new_vector;
     }
-#line 2642 "y.tab.c"
+#line 2574 "y.tab.c"
     break;
 
   case 60:
-#line 1111 "src/grammar/dice.yacc"
+#line 1044 "src/grammar/dice.yacc"
              {
         vec new_vec;
         initialize_vector(&new_vec, NUMERIC, 1);
         new_vec.content[0] = 1;
         (yyval.values) = new_vec;
     }
-#line 2653 "y.tab.c"
+#line 2585 "y.tab.c"
     break;
 
   case 61:
-#line 1118 "src/grammar/dice.yacc"
+#line 1051 "src/grammar/dice.yacc"
                   {
         vec new_vec;
         initialize_vector(&new_vec, NUMERIC, 1);
         new_vec.content[0] = 0;
         (yyval.values) = new_vec;
     }
-#line 2664 "y.tab.c"
+#line 2596 "y.tab.c"
     break;
 
 
-#line 2668 "y.tab.c"
+#line 2600 "y.tab.c"
 
       default: break;
     }
@@ -2896,7 +2828,7 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 1127 "src/grammar/dice.yacc"
+#line 1060 "src/grammar/dice.yacc"
 
 /* Subroutines */
 
@@ -2910,9 +2842,8 @@ int roll(char * s){
     verbose = 0;
     YY_BUFFER_STATE buffer = yy_scan_string(s);
     yyparse();
-
     yy_delete_buffer(buffer);
-    return 0;
+    return gnoll_errno;
 }
 int roll_verbose(char * s){
     initialize();
@@ -2922,16 +2853,18 @@ int roll_verbose(char * s){
     yyparse();
 
     yy_delete_buffer(buffer);
-    return 0;
+    return gnoll_errno;
 }
 int roll_and_write(char * s, char * f){
+    gnoll_errno = 0;
     /* Write the result to file. */
     write_to_file = 1;
     output_file = f;
-    if(verbose) printf("Rolling: %s\n", s);
+    if(verbose) safe_printf("Rolling: %s\n", s);
     return roll(s);
 }
 int mock_roll(char * s, char * f, int mock_value, int quiet, int mock_const){
+    gnoll_errno = 0;
     init_mocking(mock_value, mock_const);
     verbose = !quiet;
     return roll_and_write(s, f);
@@ -2948,8 +2881,7 @@ char * concat_strings(char ** s, int num_s){
         size_total -= 1;  // no need for trailing space
     }
     char * result;
-    result = (char *)calloc(sizeof(char), (size_total+1));
-
+    result = (char *)safe_calloc(sizeof(char), (size_total+1));
     for(int i = 1; i != num_s + 1; i++){
         strcat(result, s[i]);
         if (spaces && i < num_s){
@@ -2969,26 +2901,19 @@ int main(int argc, char **str){
 int yyerror(s)
 const char *s;
 {
-    fprintf(stderr, "%s\n", s);
+    safe_fprintf(stderr, "%s\n", s);
 
     if(write_to_file){
         FILE *fp;
-        fp = fopen(output_file, "a+");
-        fprintf(fp, "%s;", s);
-        fclose(fp);
+        fp = safe_fopen(output_file, "a+");
+        safe_fprintf(fp, "%s;", s);
+        safe_fclose(fp);
     }
-    return(1);
+    return(gnoll_errno);
 
 }
 
 int yywrap(){
     return (1);
 }
-void print_err_if_present(int err_code){
-    switch(err_code){
-        case 1:{
-            printf("Negative Dice Sides not Allowed\n");
-            break;
-        }
-    }
-}
+
